#!/usr/node/bin/node
/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at http://smartos.org/CDDL
 *
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file.
 *
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * Copyright (c) 2014, Joyent, Inc. All rights reserved.
 *
 * SUMMARY:
 *
 *  This is a test/dummy driver for the getVmobjs() function. If no arguments
 *  are passed, it will output an array with all VMs. If arguments are passed
 *  the arguments are assumed to be uuids and only those VMs will be included
 *  in the output array.
 */

var async = require('/usr/node/node_modules/async');
var bunyan = require('/usr/node/node_modules/bunyan');
var getVmobjs = require('vmload').getVmobjs;

var log = bunyan.createLogger({
    level: 'debug',
    name: 'dump-vmobjs',
    streams: [
        {
            stream: process.stderr,
            level: 'debug'
        }
    ],
    serializers: bunyan.stdSerializers
});

getVmobjs(function (vm, cb) {
    var args;

    if (process.argv.length >= 3) {
        args = process.argv.slice(2);
        if (args.indexOf(vm.uuid) !== -1) {
            cb(true);
        } else {
            cb(false);
        }
    } else {
        cb(true);
    }
}, {log: log}, function (err, results) {

    if (err) {
        throw err;
    }
    console.log(JSON.stringify(results, null, 2));
});
