#!/usr/node/bin/node
/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at http://smartos.org/CDDL
 *
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file.
 *
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * Copyright (c) 2014, Joyent, Inc. All rights reserved.
 *
 * SUMMARY:
 *
 *  This is a test/dummy driver for the getZoneRecords() function. If arguments
 *  are passed, they are assumed to be VM UUIDs and this will only lookup
 *  information for those VMs. If no arguments are passed this outputs data for
 *  all VMs found on the system. Outputs is an object with a key (zone uuid) for
 *  each VM, and a value including the 'zoneadm list -pc' properties of that VM.
 */

var bunyan = require('/usr/node/node_modules/bunyan');
var getZoneRecords = require('./vmload-zoneadm').getZoneRecords;

var argv = [];
var log = bunyan.createLogger({
    level: 'debug',
    name: 'dump-zoneadm',
    streams: [
        {
            stream: process.stderr,
            level: 'debug'
        }
    ],
    serializers: bunyan.stdSerializers
});

if (process.argv.length >= 3) {
    argv = process.argv.slice(2);
}

getZoneRecords(argv.length === 1 ? argv[0] : null, {log: log},
    function (err, results) {
        if (err) {
            throw err;
        }

        if (argv.length > 1) {
            Object.keys(results).forEach(function (key) {
                if (argv.indexOf(key) === -1) {
                    delete results[key];
                }
            });
        }

        console.log(JSON.stringify(results, null, 2));
    }
);
